@@ -671,7 +671,7 @@ raise_file_descriptor_limit (BusContext      *context)
 
   if (context->initial_fd_limit == NULL)
     {
-      bus_context_log (context, DBUS_SYSTEM_LOG_INFO,
+      bus_context_log (context, DBUS_SYSTEM_LOG_WARNING,
                        "%s: %s", error.name, error.message);
       dbus_error_free (&error);
       return;
@@ -686,7 +686,7 @@ raise_file_descriptor_limit (BusContext      *context)
    */
   if (!_dbus_rlimit_raise_fd_limit_if_privileged (65536, &error))
     {
-      bus_context_log (context, DBUS_SYSTEM_LOG_INFO,
+      bus_context_log (context, DBUS_SYSTEM_LOG_WARNING,
                        "%s: %s", error.name, error.message);
       dbus_error_free (&error);
       return;
@@ -1324,9 +1324,6 @@ bus_context_get_initial_fd_limit (BusContext *context)
   return context->initial_fd_limit;
 }
 
-void
-bus_context_log (BusContext *context, DBusSystemLogSeverity severity, const char *msg, ...) _DBUS_GNUC_PRINTF (3, 4);
-
 void
 bus_context_log (BusContext *context, DBusSystemLogSeverity severity, const char *msg, ...)
 {
@@ -1339,6 +1336,10 @@ bus_context_log (BusContext *context, DBusSystemLogSeverity severity, const char
       vfprintf (stderr, msg, args);
       fprintf (stderr, "\n");
       va_end (args);
+
+      if (severity == DBUS_SYSTEM_LOG_FATAL)
+        _dbus_exit (1);
+
       return;
     }
 
@@ -1373,6 +1374,49 @@ nonnull (const char *maybe_null,
   return (maybe_null ? maybe_null : if_null);
 }
 
+void
+bus_context_log_literal (BusContext            *context,
+                         DBusSystemLogSeverity  severity,
+                         const char            *msg)
+{
+  if (!context->syslog)
+    {
+      fputs (msg, stderr);
+      fputc ('\n', stderr);
+
+      if (severity == DBUS_SYSTEM_LOG_FATAL)
+        _dbus_exit (1);
+    }
+  else
+    {
+      _dbus_system_log (severity, "%s%s", nonnull (context->log_prefix, ""),
+                        msg);
+    }
+}
+
+void
+bus_context_log_and_set_error (BusContext            *context,
+                               DBusSystemLogSeverity  severity,
+                               DBusError             *error,
+                               const char            *name,
+                               const char            *msg,
+                               ...)
+{
+  DBusError stack_error = DBUS_ERROR_INIT;
+  va_list args;
+
+  va_start (args, msg);
+  _dbus_set_error_valist (&stack_error, name, msg, args);
+  va_end (args);
+
+  /* If we hit OOM while setting the error, this will syslog "out of memory"
+   * which is itself an indication that something is seriously wrong */
+  bus_context_log_literal (context, DBUS_SYSTEM_LOG_SECURITY,
+                           stack_error.message);
+
+  dbus_move_error (&stack_error, error);
+}
+
 /*
  * Log something about a message, usually that it was rejected.
  */
@@ -1431,7 +1475,7 @@ complain_about_message (BusContext     *context,
   /* If we hit OOM while setting the error, this will syslog "out of memory"
    * which is itself an indication that something is seriously wrong */
   if (log)
-    bus_context_log (context, DBUS_SYSTEM_LOG_SECURITY, "%s",
+    bus_context_log_literal (context, DBUS_SYSTEM_LOG_SECURITY,
         stack_error.message);
 
   dbus_move_error (&stack_error, error);
