@@ -21,6 +21,7 @@
  *
  */
  
+#include <config.h>
 #include <dbus/dbus-internals.h>
 #include <string.h>
 
@@ -48,7 +49,7 @@ bus_config_parser_element_name_to_type (const char *name)
     }
   else if (strcmp (name, "type") == 0)
     {
-      return ELEMENT_TYPE;
+      return ELEMENT_CONFIGTYPE;
     }
   else if (strcmp (name, "fork") == 0)
     {
@@ -122,6 +123,10 @@ bus_config_parser_element_name_to_type (const char *name)
     {
       return ELEMENT_KEEP_UMASK;
     }
+  else if (strcmp (name, "allow_anonymous") == 0)
+    {
+      return ELEMENT_ALLOW_ANONYMOUS;
+    }
   return ELEMENT_NONE;
 }
 
@@ -164,7 +169,7 @@ bus_config_parser_element_type_to_name (ElementType type)
       return "servicehelper";
     case ELEMENT_INCLUDEDIR:
       return "includedir";
-    case ELEMENT_TYPE:
+    case ELEMENT_CONFIGTYPE:
       return "type";
     case ELEMENT_SELINUX:
       return "selinux";
@@ -174,6 +179,8 @@ bus_config_parser_element_type_to_name (ElementType type)
       return "syslog";
     case ELEMENT_KEEP_UMASK:
       return "keep_umask";
+    case ELEMENT_ALLOW_ANONYMOUS:
+      return "allow_anonymous";
     }
 
   _dbus_assert_not_reached ("bad element type");
