@@ -40,6 +40,24 @@
 #include <dbus/dbus-marshal-recursive.h>
 #include <string.h>
 
+static DBusConnection *
+bus_driver_get_owner_of_name (DBusConnection *connection,
+                              const char     *name)
+{
+  BusRegistry *registry;
+  BusService *serv;
+  DBusString str;
+
+  registry = bus_connection_get_registry (connection);
+  _dbus_string_init_const (&str, name);
+  serv = bus_registry_lookup (registry, &str);
+
+  if (serv == NULL)
+    return NULL;
+
+  return bus_service_get_primary_owners_connection (serv);
+}
+
 static DBusConnection *
 bus_driver_get_conn_helper (DBusConnection  *connection,
                             DBusMessage     *message,
@@ -47,11 +65,8 @@ bus_driver_get_conn_helper (DBusConnection  *connection,
                             const char     **name_p,
                             DBusError       *error)
 {
-  const char *name;
-  BusRegistry *registry;
-  BusService *serv;
-  DBusString str;
   DBusConnection *conn;
+  const char *name;
 
   if (!dbus_message_get_args (message, error,
                               DBUS_TYPE_STRING, &name,
@@ -61,11 +76,9 @@ bus_driver_get_conn_helper (DBusConnection  *connection,
   _dbus_assert (name != NULL);
   _dbus_verbose ("asked for %s of connection %s\n", what_we_want, name);
 
-  registry = bus_connection_get_registry (connection);
-  _dbus_string_init_const (&str, name);
-  serv = bus_registry_lookup (registry, &str);
+  conn = bus_driver_get_owner_of_name (connection, name);
 
-  if (serv == NULL)
+  if (conn == NULL)
     {
       dbus_set_error (error, DBUS_ERROR_NAME_HAS_NO_OWNER,
                       "Could not get %s of name '%s': no such name",
@@ -73,15 +86,101 @@ bus_driver_get_conn_helper (DBusConnection  *connection,
       return NULL;
     }
 
-  conn = bus_service_get_primary_owners_connection (serv);
-  _dbus_assert (conn != NULL);
-
   if (name_p != NULL)
     *name_p = name;
 
   return conn;
 }
 
+/*
+ * Log a security warning and set error unless the uid of the connection
+ * is either the uid of this process, or on Unix, uid 0 (root).
+ *
+ * This is intended to be a second line of defence after <deny> rules,
+ * to mitigate incorrect system bus security policy configuration files
+ * like the ones in CVE-2014-8148 and CVE-2014-8156, and (if present)
+ * LSM rules; so it doesn't need to be perfect, but as long as we have
+ * potentially dangerous functionality in the system bus, it does need
+ * to exist.
+ */
+static dbus_bool_t
+bus_driver_check_caller_is_privileged (DBusConnection *connection,
+                                       BusTransaction *transaction,
+                                       DBusMessage    *message,
+                                       DBusError      *error)
+{
+#ifdef DBUS_UNIX
+  unsigned long uid;
+
+  if (!dbus_connection_get_unix_user (connection, &uid))
+    {
+      const char *method = dbus_message_get_member (message);
+
+      bus_context_log_and_set_error (bus_transaction_get_context (transaction),
+          DBUS_SYSTEM_LOG_SECURITY, error, DBUS_ERROR_ACCESS_DENIED,
+          "rejected attempt to call %s by unknown uid", method);
+      return FALSE;
+    }
+
+  /* I'm writing it in this slightly strange form so that it's more
+   * obvious that this security-sensitive code is correct.
+   */
+  if (_dbus_unix_user_is_process_owner (uid))
+    {
+      /* OK */
+    }
+  else if (uid == 0)
+    {
+      /* OK */
+    }
+  else
+    {
+      const char *method = dbus_message_get_member (message);
+
+      bus_context_log_and_set_error (bus_transaction_get_context (transaction),
+          DBUS_SYSTEM_LOG_SECURITY, error, DBUS_ERROR_ACCESS_DENIED,
+          "rejected attempt to call %s by uid %lu", method, uid);
+      return FALSE;
+    }
+
+  return TRUE;
+#elif defined(DBUS_WIN)
+  char *windows_sid = NULL;
+  dbus_bool_t ret = FALSE;
+
+  if (!dbus_connection_get_windows_user (connection, &windows_sid))
+    {
+      const char *method = dbus_message_get_member (message);
+
+      bus_context_log_and_set_error (bus_transaction_get_context (transaction),
+          DBUS_SYSTEM_LOG_SECURITY, error, DBUS_ERROR_ACCESS_DENIED,
+          "rejected attempt to call %s by unknown uid", method);
+      goto out;
+    }
+
+  if (!_dbus_windows_user_is_process_owner (windows_sid))
+    {
+      const char *method = dbus_message_get_member (message);
+
+      bus_context_log_and_set_error (bus_transaction_get_context (transaction),
+          DBUS_SYSTEM_LOG_SECURITY, error, DBUS_ERROR_ACCESS_DENIED,
+          "rejected attempt to call %s by uid %s", method, windows_sid);
+      goto out;
+    }
+
+  ret = TRUE;
+out:
+  dbus_free (windows_sid);
+  return ret;
+#else
+  /* make sure we fail closed in the hypothetical case that we are neither
+   * Unix nor Windows */
+  dbus_set_error (error, DBUS_ERROR_ACCESS_DENIED,
+      "please teach bus/driver.c how uids work on this platform");
+  return FALSE;
+#endif
+}
+
 static dbus_bool_t bus_driver_send_welcome_message (DBusConnection *connection,
                                                     DBusMessage    *hello_message,
                                                     BusTransaction *transaction,
@@ -128,6 +227,9 @@ bus_driver_send_service_owner_changed (const char     *service_name,
 
   _dbus_assert (dbus_message_has_signature (message, "sss"));
 
+  if (!bus_transaction_capture (transaction, NULL, message))
+    goto oom;
+
   retval = bus_dispatch_matches (transaction, NULL, NULL, message, error);
   dbus_message_unref (message);
 
@@ -884,35 +986,12 @@ bus_driver_handle_update_activation_environment (DBusConnection *connection,
 #ifdef DBUS_UNIX
     {
       /* UpdateActivationEnvironment is basically a recipe for privilege
-      * escalation so let's be extra-careful: do not allow the sysadmin
-      * to shoot themselves in the foot. */
-      unsigned long uid;
-
-      if (!dbus_connection_get_unix_user (connection, &uid))
-        {
-          bus_context_log (bus_transaction_get_context (transaction),
-              DBUS_SYSTEM_LOG_SECURITY,
-              "rejected attempt to call UpdateActivationEnvironment by "
-              "unknown uid");
-          dbus_set_error (error, DBUS_ERROR_ACCESS_DENIED,
-              "rejected attempt to call UpdateActivationEnvironment by "
-              "unknown uid");
-          return FALSE;
-        }
-
-      /* On the system bus, we could in principle allow uid 0 to call
-       * UpdateActivationEnvironment; but they should know better anyway,
-       * and our default system.conf has always forbidden it */
-      if (!_dbus_unix_user_is_process_owner (uid))
-        {
-          bus_context_log (bus_transaction_get_context (transaction),
-              DBUS_SYSTEM_LOG_SECURITY,
-              "rejected attempt to call UpdateActivationEnvironment by uid %lu",
-              uid);
-          dbus_set_error (error, DBUS_ERROR_ACCESS_DENIED,
-              "rejected attempt to call UpdateActivationEnvironment");
-          return FALSE;
-        }
+       * escalation so let's be extra-careful: do not allow the sysadmin
+       * to shoot themselves in the foot.
+       */
+      if (!bus_driver_check_caller_is_privileged (connection, transaction,
+                                                  message, error))
+        return FALSE;
     }
 #endif
 
@@ -1613,8 +1692,6 @@ bus_driver_handle_get_connection_credentials (DBusConnection *connection,
       goto oom;
     }
 
-  dbus_message_unref (reply);
-
   return TRUE;
 
  oom:
@@ -1724,6 +1801,117 @@ bus_driver_handle_get_id (DBusConnection *connection,
   return FALSE;
 }
 
+static dbus_bool_t
+bus_driver_handle_become_monitor (DBusConnection *connection,
+                                  BusTransaction *transaction,
+                                  DBusMessage    *message,
+                                  DBusError      *error)
+{
+  char **match_rules = NULL;
+  BusMatchRule *rule;
+  DBusList *rules = NULL;
+  DBusList *iter;
+  DBusString str;
+  int i;
+  int n_match_rules;
+  dbus_uint32_t flags;
+  dbus_bool_t ret = FALSE;
+
+  _DBUS_ASSERT_ERROR_IS_CLEAR (error);
+
+  if (!bus_driver_check_message_is_for_us (message, error))
+    goto out;
+
+  if (!bus_driver_check_caller_is_privileged (connection, transaction,
+                                              message, error))
+    goto out;
+
+  if (!dbus_message_get_args (message, error,
+        DBUS_TYPE_ARRAY, DBUS_TYPE_STRING, &match_rules, &n_match_rules,
+        DBUS_TYPE_UINT32, &flags,
+        DBUS_TYPE_INVALID))
+    goto out;
+
+  if (flags != 0)
+    {
+      dbus_set_error (error, DBUS_ERROR_INVALID_ARGS,
+          "BecomeMonitor does not support any flags yet");
+      goto out;
+    }
+
+  /* Special case: a zero-length array becomes [""] */
+  if (n_match_rules == 0)
+    {
+      match_rules = dbus_malloc (2 * sizeof (char *));
+
+      if (match_rules == NULL)
+        {
+          BUS_SET_OOM (error);
+          goto out;
+        }
+
+      match_rules[0] = _dbus_strdup ("");
+
+      if (match_rules[0] == NULL)
+        {
+          BUS_SET_OOM (error);
+          goto out;
+        }
+
+      match_rules[1] = NULL;
+      n_match_rules = 1;
+    }
+
+  for (i = 0; i < n_match_rules; i++)
+    {
+      _dbus_string_init_const (&str, match_rules[i]);
+      rule = bus_match_rule_parse (connection, &str, error);
+
+      if (rule == NULL)
+        {
+          BUS_SET_OOM (error);
+          goto out;
+        }
+
+      /* monitors always eavesdrop */
+      bus_match_rule_set_client_is_eavesdropping (rule, TRUE);
+
+      if (!_dbus_list_append (&rules, rule))
+        {
+          BUS_SET_OOM (error);
+          bus_match_rule_unref (rule);
+          goto out;
+        }
+    }
+
+  /* Send the ack before we remove the rule, since the ack is undone
+   * on transaction cancel, but becoming a monitor isn't.
+   */
+  if (!send_ack_reply (connection, transaction, message, error))
+    goto out;
+
+  if (!bus_connection_be_monitor (connection, transaction, &rules, error))
+    goto out;
+
+  ret = TRUE;
+
+out:
+  if (ret)
+    _DBUS_ASSERT_ERROR_IS_CLEAR (error);
+  else
+    _DBUS_ASSERT_ERROR_IS_SET (error);
+
+  for (iter = _dbus_list_get_first_link (&rules);
+      iter != NULL;
+      iter = _dbus_list_get_next_link (&rules, iter))
+    bus_match_rule_unref (iter->data);
+
+  _dbus_list_clear (&rules);
+
+  dbus_free_string_array (match_rules);
+  return ret;
+}
+
 typedef struct
 {
   const char *name;
@@ -1825,10 +2013,16 @@ static const MessageHandler introspectable_message_handlers[] = {
   { NULL, NULL, NULL, NULL }
 };
 
+static const MessageHandler monitoring_message_handlers[] = {
+  { "BecomeMonitor", "asu", "", bus_driver_handle_become_monitor },
+  { NULL, NULL, NULL, NULL }
+};
+
 #ifdef DBUS_ENABLE_STATS
 static const MessageHandler stats_message_handlers[] = {
   { "GetStats", "", "a{sv}", bus_stats_handle_get_stats },
   { "GetConnectionStats", "s", "a{sv}", bus_stats_handle_get_connection_stats },
+  { "GetAllMatchRules", "", "a{sas}", bus_stats_handle_get_all_match_rules },
   { NULL, NULL, NULL, NULL }
 };
 #endif
@@ -1855,6 +2049,7 @@ static InterfaceHandler interface_handlers[] = {
     "      <arg type=\"s\"/>\n"
     "    </signal>\n" },
   { DBUS_INTERFACE_INTROSPECTABLE, introspectable_message_handlers, NULL },
+  { DBUS_INTERFACE_MONITORING, monitoring_message_handlers, NULL },
 #ifdef DBUS_ENABLE_STATS
   { BUS_INTERFACE_STATS, stats_message_handlers, NULL },
 #endif
@@ -2053,8 +2248,26 @@ bus_driver_handle_message (DBusConnection *connection,
   if (dbus_message_is_signal (message, "org.freedesktop.systemd1.Activator", "ActivationFailure"))
     {
       BusContext *context;
+      DBusConnection *systemd;
 
       context = bus_connection_get_context (connection);
+      systemd = bus_driver_get_owner_of_name (connection,
+          "org.freedesktop.systemd1");
+
+      if (systemd != connection)
+        {
+          const char *attacker;
+
+          attacker = bus_connection_get_name (connection);
+          bus_context_log (context, DBUS_SYSTEM_LOG_SECURITY,
+                           "Ignoring forged ActivationFailure message from "
+                           "connection %s (%s)",
+                           attacker ? attacker : "(unauthenticated)",
+                           bus_connection_get_loginfo (connection));
+          /* ignore it */
+          return TRUE;
+        }
+
       return dbus_activation_systemd_failure(bus_context_get_activation(context), message);
     }
 
