@@ -75,6 +75,9 @@ void          _dbus_string_init_const_len        (DBusString        *str,
                                                   int                len);
 dbus_bool_t   _dbus_string_init_preallocated     (DBusString        *str,
                                                   int                allocate_size);
+
+dbus_bool_t _dbus_string_init_from_string        (DBusString        *str,
+                                                  const DBusString  *from);
 void          _dbus_string_free                  (DBusString        *str);
 void          _dbus_string_lock                  (DBusString        *str);
 dbus_bool_t   _dbus_string_compact               (DBusString        *str,
