@@ -518,10 +518,10 @@ _dbus_close_socket (int        fd,
  * on exec. Should be called for all file
  * descriptors in D-Bus code.
  *
- * @param handle the Windows HANDLE
+ * @param handle the Windows HANDLE (a SOCKET is also OK)
  */
-void
-_dbus_fd_set_close_on_exec (intptr_t handle)
+static void
+_dbus_win_handle_set_close_on_exec (HANDLE handle)
 {
   if ( !SetHandleInformation( (HANDLE) handle,
                         HANDLE_FLAG_INHERIT | HANDLE_FLAG_PROTECT_FROM_CLOSE,
@@ -1605,7 +1605,8 @@ _dbus_connect_tcp_socket_with_nonce (const char     *host,
         }
     }
 
-  _dbus_fd_set_close_on_exec (fd);
+  /* Every SOCKET is also a HANDLE. */
+  _dbus_win_handle_set_close_on_exec ((HANDLE) fd);
 
   if (!_dbus_set_fd_nonblocking (fd, error))
     {
@@ -1665,7 +1666,7 @@ _dbus_listen_tcp_socket (const char     *host,
   _DBUS_ZERO (hints);
 
   if (!family)
-    hints.ai_family = AF_UNSPEC;
+    hints.ai_family = AF_INET;
   else if (!strcmp(family, "ipv4"))
     hints.ai_family = AF_INET;
   else if (!strcmp(family, "ipv6"))
@@ -1762,9 +1763,12 @@ _dbus_listen_tcp_socket (const char     *host,
             {
               mysockaddr_gen addr;
               socklen_t addrlen = sizeof(addr);
-              char portbuf[10];
+              char portbuf[NI_MAXSERV];
 
-              if (getsockname(fd, &addr.Address, &addrlen) == SOCKET_ERROR)
+              if (getsockname(fd, &addr.Address, &addrlen) == SOCKET_ERROR ||
+                (res = getnameinfo (&addr.Address, addrlen, NULL, 0,
+                                    portbuf, sizeof(portbuf),
+                                    NI_NUMERICSERV)) != 0)
                 {
                   DBUS_SOCKET_SET_ERRNO ();
                   dbus_set_error (error, _dbus_error_from_errno (errno),
@@ -1772,10 +1776,6 @@ _dbus_listen_tcp_socket (const char     *host,
                                   host ? host : "*", port, _dbus_strerror_from_errno());
                   goto failed;
                 }
-              if (addr.AddressIn.sin_family = AF_INET)
-                  snprintf( portbuf, sizeof( portbuf ) - 1, "%d", ntohs(addr.AddressIn.sin_port) );
-              else
-                  snprintf( portbuf, sizeof( portbuf ) - 1, "%d", ntohs(addr.AddressIn6.sin6_port) );
               if (!_dbus_string_append(retport, portbuf))
                 {
                   dbus_set_error (error, DBUS_ERROR_NO_MEMORY, NULL);
@@ -1815,7 +1815,7 @@ _dbus_listen_tcp_socket (const char     *host,
 
   for (i = 0 ; i < nlisten_fd ; i++)
     {
-      _dbus_fd_set_close_on_exec (listen_fd[i]);
+      _dbus_win_handle_set_close_on_exec ((HANDLE) listen_fd[i]);
       if (!_dbus_set_fd_nonblocking (listen_fd[i], error))
         {
           goto failed;
@@ -3264,12 +3264,12 @@ _dbus_flush_caches (void)
  * See if errno is EAGAIN or EWOULDBLOCK (this has to be done differently
  * for Winsock so is abstracted)
  *
- * @returns #TRUE if errno == EAGAIN or errno == EWOULDBLOCK
+ * @returns #TRUE if e == EAGAIN or e == EWOULDBLOCK
  */
 dbus_bool_t
-_dbus_get_is_errno_eagain_or_ewouldblock (void)
+_dbus_get_is_errno_eagain_or_ewouldblock (int e)
 {
-  return errno == WSAEWOULDBLOCK;
+  return e == WSAEWOULDBLOCK;
 }
 
 /**
@@ -3736,6 +3736,18 @@ _dbus_check_setuid (void)
   return FALSE;
 }
 
+int
+_dbus_save_socket_errno (void)
+{
+  return errno;
+}
+
+void
+_dbus_restore_socket_errno (int saved_errno)
+{
+  _dbus_win_set_errno (saved_errno);
+}
+
 /** @} end of sysdeps-win */
 /* tests in dbus-sysdeps-util.c */
 
