@@ -26,22 +26,37 @@
 
 #include <config.h>
 
-#include <glib.h>
+#include "test-utils-glib.h"
 
-#include <dbus/dbus.h>
-
-#include <string.h>
-
-#ifdef DBUS_WIN
-# include <io.h>
-# include <windows.h>
-#else
-# include <signal.h>
+#ifdef DBUS_UNIX
 # include <unistd.h>
 # include <sys/types.h>
 #endif
 
-#include "test-utils.h"
+/* Platforms where we know that credentials-passing passes both the
+ * uid and the pid. Please keep these in alphabetical order.
+ *
+ * These platforms should #error in _dbus_read_credentials_socket()
+ * if we didn't detect their flavour of credentials-passing, since that
+ * would be a regression.
+ */
+#if defined(__FreeBSD__) || defined(__FreeBSD_kernel__) || \
+  defined(__linux__) || \
+  defined(__NetBSD__) || \
+  defined(__OpenBSD__)
+# define UNIX_USER_SHOULD_WORK
+# define PID_SHOULD_WORK
+#endif
+
+/* Platforms where we know that credentials-passing passes the
+ * uid, but not necessarily the pid. Again, alphabetical order please.
+ *
+ * These platforms should also #error in _dbus_read_credentials_socket()
+ * if we didn't detect their flavour of credentials-passing.
+ */
+#if 0 /* defined(__your_platform_here__) */
+# define UNIX_USER_SHOULD_WORK
+#endif
 
 typedef struct {
     gboolean skip;
@@ -57,111 +72,27 @@ typedef struct {
 
     DBusConnection *right_conn;
     gboolean right_conn_echo;
+    gboolean wait_forever_called;
 } Fixture;
 
-#define assert_no_error(e) _assert_no_error (e, __FILE__, __LINE__)
-static void
-_assert_no_error (const DBusError *e,
-    const char *file,
-    int line)
-{
-  if (G_UNLIKELY (dbus_error_is_set (e)))
-    g_error ("%s:%d: expected success but got error: %s: %s",
-        file, line, e->name, e->message);
-}
-
-static gchar *
-spawn_dbus_daemon (gchar *binary,
-    gchar *configuration,
-    GPid *daemon_pid)
-{
-  GError *error = NULL;
-  GString *address;
-  gint address_fd;
-  gchar *argv[] = {
-      binary,
-      configuration,
-      "--nofork",
-      "--print-address=1", /* stdout */
-      NULL
-  };
-
-  g_spawn_async_with_pipes (NULL, /* working directory */
-      argv,
-      NULL, /* envp */
-      G_SPAWN_DO_NOT_REAP_CHILD | G_SPAWN_SEARCH_PATH,
-      NULL, /* child_setup */
-      NULL, /* user data */
-      daemon_pid,
-      NULL, /* child's stdin = /dev/null */
-      &address_fd,
-      NULL, /* child's stderr = our stderr */
-      &error);
-  g_assert_no_error (error);
-
-  address = g_string_new (NULL);
-
-  /* polling until the dbus-daemon writes out its address is a bit stupid,
-   * but at least it's simple, unlike dbus-launch... in principle we could
-   * use select() here, but life's too short */
-  while (1)
-    {
-      gssize bytes;
-      gchar buf[4096];
-      gchar *newline;
-
-      bytes = read (address_fd, buf, sizeof (buf));
-
-      if (bytes > 0)
-        g_string_append_len (address, buf, bytes);
-
-      newline = strchr (address->str, '\n');
-
-      if (newline != NULL)
-        {
-          if ((newline > address->str) && ('\r' == newline[-1]))
-            newline -= 1;
-          g_string_truncate (address, newline - address->str);
-          break;
-        }
-
-      g_usleep (G_USEC_PER_SEC / 10);
-    }
-
-  return g_string_free (address, FALSE);
-}
-
-static DBusConnection *
-connect_to_bus (Fixture *f,
-    const gchar *address)
-{
-  DBusConnection *conn;
-  DBusError error = DBUS_ERROR_INIT;
-  dbus_bool_t ok;
-
-  conn = dbus_connection_open_private (address, &error);
-  assert_no_error (&error);
-  g_assert (conn != NULL);
-
-  ok = dbus_bus_register (conn, &error);
-  assert_no_error (&error);
-  g_assert (ok);
-  g_assert (dbus_bus_get_unique_name (conn) != NULL);
-
-  test_connection_setup (f->ctx, conn);
-  return conn;
-}
-
 static DBusHandlerResult
 echo_filter (DBusConnection *connection,
     DBusMessage *message,
     void *user_data)
 {
+  Fixture *f = user_data;
   DBusMessage *reply;
 
   if (dbus_message_get_type (message) != DBUS_MESSAGE_TYPE_METHOD_CALL)
     return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 
+  /* WaitForever() never replies, emulating a service that has got stuck */
+  if (dbus_message_is_method_call (message, "com.example", "WaitForever"))
+    {
+      f->wait_forever_called = TRUE;
+      return DBUS_HANDLER_RESULT_HANDLED;
+    }
+
   reply = dbus_message_new_method_return (message);
 
   if (reply == NULL)
@@ -186,78 +117,31 @@ setup (Fixture *f,
     gconstpointer context)
 {
   const Config *config = context;
-  gchar *dbus_daemon;
-  gchar *arg;
   gchar *address;
 
   f->ctx = test_main_context_get ();
   f->ge = NULL;
   dbus_error_init (&f->e);
 
-  if (config != NULL && config->config_file != NULL)
-    {
-      if (g_getenv ("DBUS_TEST_DAEMON_ADDRESS") != NULL)
-        {
-          g_message ("SKIP: cannot use DBUS_TEST_DAEMON_ADDRESS for "
-              "unusally-configured dbus-daemon");
-          f->skip = TRUE;
-          return;
-        }
+  address = test_get_dbus_daemon (config ? config->config_file : NULL,
+                                  TEST_USER_ME,
+                                  &f->daemon_pid);
 
-      if (g_getenv ("DBUS_TEST_DATA") == NULL)
-        {
-          g_message ("SKIP: set DBUS_TEST_DATA to a directory containing %s",
-              config->config_file);
-          f->skip = TRUE;
-          return;
-        }
-
-      arg = g_strdup_printf (
-          "--config-file=%s/%s",
-          g_getenv ("DBUS_TEST_DATA"), config->config_file);
-    }
-  else if (g_getenv ("DBUS_TEST_SYSCONFDIR") != NULL)
+  if (address == NULL)
     {
-      arg = g_strdup_printf ("--config-file=%s/dbus-1/session.conf",
-          g_getenv ("DBUS_TEST_SYSCONFDIR"));
-    }
-  else if (g_getenv ("DBUS_TEST_DATA") != NULL)
-    {
-      arg = g_strdup_printf (
-          "--config-file=%s/valid-config-files/session.conf",
-          g_getenv ("DBUS_TEST_DATA"));
-    }
-  else
-    {
-      arg = g_strdup ("--session");
+      f->skip = TRUE;
+      return;
     }
 
-  dbus_daemon = g_strdup (g_getenv ("DBUS_TEST_DAEMON"));
-
-  if (dbus_daemon == NULL)
-    dbus_daemon = g_strdup ("dbus-daemon");
-
-  if (g_getenv ("DBUS_TEST_DAEMON_ADDRESS") != NULL)
-    {
-      address = g_strdup (g_getenv ("DBUS_TEST_DAEMON_ADDRESS"));
-    }
-  else
-    {
-      address = spawn_dbus_daemon (dbus_daemon, arg, &f->daemon_pid);
-    }
-
-  g_free (dbus_daemon);
-  g_free (arg);
-
-  f->left_conn = connect_to_bus (f, address);
-  f->right_conn = connect_to_bus (f, address);
+  f->left_conn = test_connect_to_bus (f->ctx, address);
+  f->right_conn = test_connect_to_bus (f->ctx, address);
   g_free (address);
 }
 
 static void
 add_echo_filter (Fixture *f)
 {
-  if (!dbus_connection_add_filter (f->right_conn, echo_filter, NULL, NULL))
+  if (!dbus_connection_add_filter (f->right_conn, echo_filter, f, NULL))
     g_error ("OOM");
 
   f->right_conn_echo = TRUE;
@@ -333,13 +217,77 @@ test_echo (Fixture *f,
 }
 
 static void
-pending_call_store_reply (DBusPendingCall *pc,
-    void *data)
+test_no_reply (Fixture *f,
+    gconstpointer context)
 {
-  DBusMessage **message_p = data;
+  const Config *config = context;
+  DBusMessage *m;
+  DBusPendingCall *pc;
+  DBusMessage *reply = NULL;
+  enum { TIMEOUT, DISCONNECT } mode;
+  gboolean ok;
 
-  *message_p = dbus_pending_call_steal_reply (pc);
-  g_assert (*message_p != NULL);
+  if (f->skip)
+    return;
+
+  g_test_bug ("76112");
+
+  if (config != NULL && config->config_file != NULL)
+    mode = TIMEOUT;
+  else
+    mode = DISCONNECT;
+
+  m = dbus_message_new_method_call (
+      dbus_bus_get_unique_name (f->right_conn), "/",
+      "com.example", "WaitForever");
+
+  add_echo_filter (f);
+
+  if (m == NULL)
+    g_error ("OOM");
+
+  if (!dbus_connection_send_with_reply (f->left_conn, m, &pc,
+                                        DBUS_TIMEOUT_INFINITE) ||
+      pc == NULL)
+    g_error ("OOM");
+
+  if (dbus_pending_call_get_completed (pc))
+    test_pending_call_store_reply (pc, &reply);
+  else if (!dbus_pending_call_set_notify (pc, test_pending_call_store_reply,
+        &reply, NULL))
+    g_error ("OOM");
+
+  dbus_pending_call_unref (pc);
+  dbus_message_unref (m);
+
+  if (mode == DISCONNECT)
+    {
+      while (!f->wait_forever_called)
+        test_main_context_iterate (f->ctx, TRUE);
+
+      dbus_connection_remove_filter (f->right_conn, echo_filter, f);
+      dbus_connection_close (f->right_conn);
+      dbus_connection_unref (f->right_conn);
+      f->right_conn = NULL;
+    }
+
+  while (reply == NULL)
+    test_main_context_iterate (f->ctx, TRUE);
+
+  /* using inefficient string comparison for better assertion message */
+  g_assert_cmpstr (
+      dbus_message_type_to_string (dbus_message_get_type (reply)), ==,
+      dbus_message_type_to_string (DBUS_MESSAGE_TYPE_ERROR));
+  ok = dbus_set_error_from_message (&f->e, reply);
+  g_assert (ok);
+  g_assert_cmpstr (f->e.name, ==, DBUS_ERROR_NO_REPLY);
+
+  if (mode == DISCONNECT)
+    g_assert_cmpstr (f->e.message, ==,
+        "Message recipient disconnected from message bus without replying");
+  else
+    g_assert_cmpstr (f->e.message, ==,
+        "Message did not receive a reply (timeout by message bus)");
 }
 
 static void
@@ -377,8 +325,8 @@ test_creds (Fixture *f,
   m = NULL;
 
   if (dbus_pending_call_get_completed (pc))
-    pending_call_store_reply (pc, &m);
-  else if (!dbus_pending_call_set_notify (pc, pending_call_store_reply,
+    test_pending_call_store_reply (pc, &m);
+  else if (!dbus_pending_call_set_notify (pc, test_pending_call_store_reply,
                                           &m, NULL))
     g_error ("OOM");
 
@@ -445,8 +393,11 @@ test_creds (Fixture *f,
       dbus_message_iter_next (&arr_iter);
     }
 
-#ifdef G_OS_UNIX
+#ifdef UNIX_USER_SHOULD_WORK
   g_assert (seen & SEEN_UNIX_USER);
+#endif
+
+#ifdef PID_SHOULD_WORK
   g_assert (seen & SEEN_PID);
 #endif
 
@@ -457,6 +408,72 @@ test_creds (Fixture *f,
 #endif
 }
 
+static void
+test_processid (Fixture *f,
+    gconstpointer context)
+{
+  const char *unique = dbus_bus_get_unique_name (f->left_conn);
+  DBusMessage *m = dbus_message_new_method_call (DBUS_SERVICE_DBUS,
+      DBUS_PATH_DBUS, DBUS_INTERFACE_DBUS, "GetConnectionUnixProcessID");
+  DBusPendingCall *pc;
+  DBusError error = DBUS_ERROR_INIT;
+  guint32 pid;
+
+  if (m == NULL)
+    g_error ("OOM");
+
+  if (!dbus_message_append_args (m,
+        DBUS_TYPE_STRING, &unique,
+        DBUS_TYPE_INVALID))
+    g_error ("OOM");
+
+  if (!dbus_connection_send_with_reply (f->left_conn, m, &pc,
+                                        DBUS_TIMEOUT_USE_DEFAULT) ||
+      pc == NULL)
+    g_error ("OOM");
+
+  dbus_message_unref (m);
+  m = NULL;
+
+  if (dbus_pending_call_get_completed (pc))
+    test_pending_call_store_reply (pc, &m);
+  else if (!dbus_pending_call_set_notify (pc, test_pending_call_store_reply,
+                                          &m, NULL))
+    g_error ("OOM");
+
+  while (m == NULL)
+    test_main_context_iterate (f->ctx, TRUE);
+
+  if (dbus_message_get_args (m, &error,
+        DBUS_TYPE_UINT32, &pid,
+        DBUS_TYPE_INVALID))
+    {
+      g_assert_cmpstr (dbus_message_get_signature (m), ==, "u");
+      test_assert_no_error (&error);
+
+      g_message ("GetConnectionUnixProcessID returned %u", pid);
+
+#ifdef G_OS_UNIX
+      g_assert_cmpuint (pid, ==, getpid ());
+#elif defined(G_OS_WIN32)
+      g_assert_cmpuint (pid, ==, GetCurrentProcessId ());
+#else
+      g_assert_not_reached ();
+#endif
+    }
+  else
+    {
+      g_assert_cmpstr (error.name, ==, DBUS_ERROR_UNIX_PROCESS_ID_UNKNOWN);
+
+#ifdef PID_SHOULD_WORK
+      g_error ("Expected pid to be passed, but got %s: %s",
+          error.name, error.message);
+#endif
+
+      dbus_error_free (&error);
+    }
+}
+
 static void
 test_canonical_path_uae (Fixture *f,
     gconstpointer context)
@@ -487,8 +504,8 @@ test_canonical_path_uae (Fixture *f,
   m = NULL;
 
   if (dbus_pending_call_get_completed (pc))
-    pending_call_store_reply (pc, &m);
-  else if (!dbus_pending_call_set_notify (pc, pending_call_store_reply,
+    test_pending_call_store_reply (pc, &m);
+  else if (!dbus_pending_call_set_notify (pc, test_pending_call_store_reply,
                                           &m, NULL))
     g_error ("OOM");
 
@@ -525,8 +542,8 @@ test_canonical_path_uae (Fixture *f,
   m = NULL;
 
   if (dbus_pending_call_get_completed (pc))
-    pending_call_store_reply (pc, &m);
-  else if (!dbus_pending_call_set_notify (pc, pending_call_store_reply,
+    test_pending_call_store_reply (pc, &m);
+  else if (!dbus_pending_call_set_notify (pc, test_pending_call_store_reply,
                                           &m, NULL))
     g_error ("OOM");
 
@@ -560,7 +577,7 @@ teardown (Fixture *f,
     {
       if (f->right_conn_echo)
         {
-          dbus_connection_remove_filter (f->right_conn, echo_filter, NULL);
+          dbus_connection_remove_filter (f->right_conn, echo_filter, f);
           f->right_conn_echo = FALSE;
         }
 
@@ -571,12 +588,7 @@ teardown (Fixture *f,
 
   if (f->daemon_pid != 0)
     {
-#ifdef DBUS_WIN
-      TerminateProcess (f->daemon_pid, 1);
-#else
-      kill (f->daemon_pid, SIGTERM);
-#endif
-
+      test_kill_pid (f->daemon_pid);
       g_spawn_close_pid (f->daemon_pid);
       f->daemon_pid = 0;
     }
@@ -588,17 +600,25 @@ static Config limited_config = {
     "34393", 10000, "valid-config-files/incoming-limit.conf"
 };
 
+static Config finite_timeout_config = {
+    NULL, 1, "valid-config-files/finite-timeout.conf"
+};
+
 int
 main (int argc,
     char **argv)
 {
-  g_test_init (&argc, &argv, NULL);
-  g_test_bug_base ("https://bugs.freedesktop.org/show_bug.cgi?id=");
+  test_init (&argc, &argv);
 
   g_test_add ("/echo/session", Fixture, NULL, setup, test_echo, teardown);
   g_test_add ("/echo/limited", Fixture, &limited_config,
       setup, test_echo, teardown);
+  g_test_add ("/no-reply/disconnect", Fixture, NULL,
+      setup, test_no_reply, teardown);
+  g_test_add ("/no-reply/timeout", Fixture, &finite_timeout_config,
+      setup, test_no_reply, teardown);
   g_test_add ("/creds", Fixture, NULL, setup, test_creds, teardown);
+  g_test_add ("/processid", Fixture, NULL, setup, test_processid, teardown);
   g_test_add ("/canonical-path/uae", Fixture, NULL,
       setup, test_canonical_path_uae, teardown);
 
